from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction
from sybil_engine.utils.file_loader import load_abi

abi = load_abi("resources/abi/socialscan.json")


class SocialScanContract(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    @evm_transaction
    def set_approval_for_all(self, account, operator, approved):
        txn_params = self.build_generic_data(account.address, False)

        contract_txn = self.contract.functions.setApprovalForAll(
            operator, approved
        ).build_transaction(txn_params)

        return contract_txn



from sybil_engine.contract.contract import Contract
from sybil_engine.utils.file_loader import load_abi

abi = load_abi("resources/abi/compound_v3.json")


class CompoundV3Contract(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    def user_collateral(self, account, token_address):
        return self.contract.functions.userCollateral(
            account.address,
            token_address
        ).call()[0]



from sybil_engine.contract.contract import Contract
from sybil_engine.contract.transaction_executor import evm_transaction
from sybil_engine.utils.file_loader import load_abi

abi = load_abi("resources/abi/dmail.json")


class DmailSend(Contract):
    def __init__(self, contract_address, web3):
        super().__init__(contract_address, web3, abi)

    @evm_transaction
    def send_mail(self, account, to, subject):
        sender = account.address

        txn_params = self.build_generic_data(sender, set_contract_address=False)

        return self.contract.functions.send_mail(to, subject).build_transaction(txn_params)



from sybil_engine.data.contracts import get_contracts_for_chain
from sybil_engine.data.networks import get_chain_instance
from sybil_engine.utils.web3_utils import init_web3

from web3_wizzard_lib.core.contract.alienswap_contract import AlienSwapContract
from web3_wizzard_lib.core.utils.sub_module import SubModule


class AlienSwap(SubModule):
    module_name = 'ALIENSWAP_LINEA'

    def execute(self, account, chain='LINEA'):
        chain_instance = get_chain_instance(chain)
        web3 = init_web3(chain_instance, account.proxy)

        contract_address = get_contracts_for_chain(chain)['ALIENSWAP_LINEA']
        as_match_nft = AlienSwapContract(contract_address, web3)

        as_match_nft.purchase(account, 1)

    def log(self):
        return "ALIENSWAP LINEA NFT"



from sybil_engine.module.module import Module
from sybil_engine.utils.utils import ConfigurationException
from sybil_engine.utils.validation_utils import validate_amount_interval, validate_chain

from web3_wizzard_lib.core.modules.bridge.scroll_bridge_sub import ScrollBridgeSubModule


class ScrollBridgeModule(Module):
    module_name = 'SCROLL_BRIDGE'
    module_config = 'scroll_bridge'

    supported_tokens = ['NATIVE']
    supported_chains = ['ETH_MAINNET', 'SCROLL']

    def execute(self, bridge_amount_interval, from_chain, to_chain, token, account):
        ScrollBridgeSubModule().execute(
            bridge_amount_interval,
            from_chain,
            to_chain,
            token,
            self.min_native_balance,
            account
        )

    def log(self):
        return "SCROLL BRIDGE"

    def parse_params(self, module_params):
        validate_amount_interval(module_params['bridge_amount_interval'])
        validate_chain(module_params['from_chain'])

        if 'token' not in module_params:
            module_params['token'] = 'NATIVE'

        if module_params['token'] not in self.supported_tokens:
            raise ConfigurationException(
                f"{module_params['token']} not supported in scroll bridge. Supported options: {self.supported_tokens}")

        if module_params['to_chain'] not in self.supported_chains or module_params[
            'to_chain'] not in self.supported_chains:
            raise ConfigurationException(
                f"{module_params['chain']} not supported in scroll bridge. Supported options: {self.supported_chains}")

        return module_params['bridge_amount_interval'], module_params['from_chain'], module_params['to_chain'], \
            module_params['token']
